%global pkg igb
%global kernel 5.10.134-17.an23
%define pkg_version 5.14.16
%define anolis_release 4

Name:        kmod-%{pkg}
Version:     %(echo %{kernel} | sed -E 's/-/~/g; s/\.(an|al)[0-9]+$//g')
Release:     %{pkg_version}~%{anolis_release}%{?dist}
Summary: Intel(R) Gigabit Ethernet Linux Driver
Source:     kmod-%{pkg}-%{pkg_version}.tar.gz
Vendor: Intel Corporation
License: GPL-2.0
ExclusiveOS: linux
Group: System Environment/Kernel
Provides: %{pkg}
URL: http://support.intel.com
BuildRoot: %{_tmppath}/%{pkg}-%{pkg_version}-root
# do not generate debugging packages by default - newer versions of rpmbuild
# may instead need:
%global debug_package %{nil}
%global __os_install_post %{nil}
# macros for finding system files to update at install time (pci.ids, pcitable)
%define find() %(for f in %*; do if [ -e $f ]; then echo $f; break; fi; done)
%define _pciids   /usr/share/pci.ids        /usr/share/hwdata/pci.ids
%define _pcitable /usr/share/kudzu/pcitable /usr/share/hwdata/pcitable /dev/null
%define pciids   /usr/share/hwdata/pci.ids
%define pcitable  %find %{_pcitable}
Requires: kernel, fileutils, findutils, gawk, bash

%define kernel_ver %{kernel}.%{_arch}

Requires:            kernel >= %{kernel}
Requires(posttrans): %{_sbindir}/depmod
Requires(postun):    %{_sbindir}/depmod
Requires(posttrans): %{_sbindir}/weak-modules
Requires(postun):    %{_sbindir}/weak-modules
Requires(posttrans): %{_bindir}/sort
Requires(postun):    %{_bindir}/sort

BuildRequires:    kernel-devel   = %{kernel}
BuildRequires:    kernel-headers = %{kernel}
BuildRequires:    elfutils-libelf-devel
BuildRequires:    gcc
BuildRequires:    kmod
BuildRequires:    make
BuildRequires:    system-rpm-config

Provides: kmod-%{pkg}-%{kernel}.%{_arch} = %{version}-%{release}
Obsoletes: kmod-%{pkg}-%{kernel}.%{_arch} < %{version}-%{release}

%description
This package contains the Intel(R) Gigabit Ethernet Linux Driver.

%prep
%setup -q -n kmod-%{pkg}-%{pkg_version}

%build
make -C src KSRC=/usr/src/kernels/%{kernel}.%{_arch}/ clean
make -C src KSRC=/usr/src/kernels/%{kernel}.%{_arch}/ -j32

%install
mkdir -p %{buildroot}/lib/modules/%{kernel}.%{_arch}/extra/drivers/net/ethernet/intel/%{pkg}
make -C src INSTALL_MOD_PATH=%{buildroot} KSRC=/usr/src/kernels/%{kernel}.%{_arch}/ MANDIR=%{_mandir} INSTALL_MOD_DIR=/extra/drivers/net/ethernet/intel/%{pkg}/ rpm
# Sign the modules(s)
%if %{?_with_modsign:1}%{!?_with_modsign:0}
cd %{buildroot}
%define __strip /bin/true
%{!?privkey: %define privkey %{_sysconfdir}/pki/SECURE-BOOT-KEY.priv}
%{!?pubkey: %define pubkey %{_sysconfdir}/pki/SECURE-BOOT-KEY.der}
%{!?_signfile: %define _signfile %{_usrsrc}/kernels/%{kernel_ver}/scripts/sign-file}
for module in `find . -type f -name *.ko`;
do
strip --strip-debug ${module}
$(KSRC=%{_usrsrc}/kernels/%{kernel_ver} %{_signfile} sha512 %{privkey} %{pubkey} ${module} > /dev/null 2>&1)
done
%endif

# Append .new to driver name to avoid conflict with kernel RPM
cd %{buildroot}
find lib -name "igb.*o" -exec mv {} {}.new \; \
         -fprintf %{_builddir}/kmod-%{pkg}-%{pkg_version}/file.list "/%p.new\n"
find lib/modules -name modules.* -exec rm -f {} \;

# Make .ko objects temporarily executable for automatic stripping
find %{buildroot}/lib/modules -type f -name \*.ko -exec chmod u+x \{\} \+

# Generate depmod.conf
%{__install} -d %{buildroot}/%{_sysconfdir}/depmod.d/
for kmod in $(find %{buildroot}/lib/modules/%{kernel}.%{_arch}/extra -type f -name \*.ko.new -printf "%%P\n" | sort)
do
    echo "override $(basename $kmod .ko) * weak-updates/$(dirname $kmod)" >> %{buildroot}/%{_sysconfdir}/depmod.d/%{pkg}.conf
    echo "override $(basename $kmod .ko) * extra/$(dirname $kmod)" >> %{buildroot}/%{_sysconfdir}/depmod.d/%{pkg}.conf
done



%clean
%{__rm} -rf %{buildroot}

%files -f file.list
%defattr(-,root,root)
%{_mandir}/man7/*
%doc COPYING
%doc README
%doc file.list
%doc pci.updates
%license licenses
%config(noreplace) %{_sysconfdir}/depmod.d/%{pkg}.conf

%post
FL="%{_docdir}/%{pkg}-%{pkg_version}/file.list
    %{_docdir}/%{pkg}/file.list"
FL=$(for d in $FL ; do if [ -e $d ]; then echo $d; break; fi;  done)

if [ -d /usr/local/lib/%{pkg} ]; then
	rm -rf /usr/local/lib/%{pkg}
fi
if [ -d /usr/local/share/%{pkg} ]; then
	rm -rf /usr/local/share/%{pkg}
fi

# Save old drivers (aka .ko and .ko.gz)
mkdir /usr/local/share/%{pkg}
cp --parents %{pciids} /usr/local/share/%{pkg}/
cp -r /usr/share/doc/%{name} /usr/share/doc/%{pkg}
echo "original pci.ids saved in /usr/local/share/%{pkg}";
if [ "%{pcitable}" != "/dev/null" ]; then
	echo "original pcitable saved in /usr/local/share/%{pkg}";
fi
for k in $(sed 's#/lib/modules/\([0-9a-zA-Z.+_-]*\).*$#\1#' $FL) ;
do
	d_drivers=/lib/modules/$k
	d_usr=/usr/local/share/%{pkg}/$k
	mkdir -p $d_usr
	cd $d_drivers; find . -name %{pkg}.*o -exec cp --parents {} $d_usr \; -exec rm -f {} \;
	cd $d_drivers; find . -name %{pkg}_*.*o -exec cp --parents {} $d_usr \; -exec rm -f {} \;
	cd $d_drivers; find . -name %{pkg}.*o.gz -exec cp --parents {} $d_usr \; -exec rm -f {} \;
	cd $d_drivers; find . -name %{pkg}_*.*o.gz -exec cp --parents {} $d_usr \; -exec rm -f {} \;
	cp --parents %{pciids} /usr/local/share/%{pkg}/
	if [ "%{pcitable}" != "/dev/null" ]; then
		cp --parents %{pcitable} /usr/local/share/%{pkg}/
	fi
done

# Add driver link
for f in $(sed 's/\.new$//' $FL) ; do
	ln -f $f.new $f
done

# Check if kernel version rpm was built on IS the same as running kernel
BK_LIST=$(sed 's#/lib/modules/\([0-9a-zA-Z.+_-]*\).*$#\1#' $FL) ;
MATCH=no
for i in $BK_LIST
do
	if [ $(uname -r) == $i ] ; then
		MATCH=yes
		break
	fi
done
if [ $MATCH == no ] ; then
	echo -n "WARNING: Running kernel is $(uname -r).  "
	echo -n "RPM supports kernels (  "
	for i in $BK_LIST
	do
		echo -n "$i  "
	done
	echo ")"
fi

LD="%{_docdir}/%{pkg}";
if [ -d %{_docdir}/%{pkg}-%{pkg_version} ]; then
	LD="%{_docdir}/%{pkg}-%{pkg_version}";
fi

#Yes, this really needs bash
bash -s %{pciids} \
	%{pcitable} \
	$LD/pci.updates \
	$LD/pci.ids.new \
	$LD/pcitable.new \
	%{pkg} \
<<"END"
#! /bin/bash
# $1 = system pci.ids file to update
# $2 = system pcitable file to update
# $3 = file with new entries in pci.ids file format
# $4 = pci.ids output file
# $5 = pcitable output file
# $6 = driver name for use in pcitable file

exec 3<$1
exec 4<$2
exec 5<$3
exec 6>$4
exec 7>$5
driver=$6
IFS=

# pattern matching strings
ID="[[:xdigit:]][[:xdigit:]][[:xdigit:]][[:xdigit:]]"
VEN="${ID}*"
DEV="	${ID}*"
SUB="		${ID}*"
TABLE_DEV="0x${ID}	0x${ID}	\"*"
TABLE_SUB="0x${ID}	0x${ID}	0x${ID}	0x${ID}	\"*"

line=
table_line=
ids_in=
table_in=
vendor=
device=
ids_device=
table_device=
subven=
ids_subven=
table_subven=
subdev=
ids_subdev=
table_subdev=
ven_str=
dev_str=
sub_str=

# force a sub-shell to fork with a new stdin
# this is needed if the shell is reading these instructions from stdin
while true
do
	# get the first line of each data file to jump start things
	exec 0<&3
	read -r ids_in
	if [ "$2" != "/dev/null" ];then
	exec 0<&4
	read -r table_in
	fi

	# outer loop reads lines from the updates file
	exec 0<&5
	while read -r line
	do
		# vendor entry
		if [[ $line == $VEN ]]
		then
			vendor=0x${line:0:4}
			ven_str=${line#${line:0:6}}
			# add entry to pci.ids
			exec 0<&3
			exec 1>&6
			while [[ $ids_in != $VEN ||
				 0x${ids_in:0:4} < $vendor ]]
			do
				echo "$ids_in"
				read -r ids_in
			done
			echo "$line"
			if [[ 0x${ids_in:0:4} == $vendor ]]
			then
				read -r ids_in
			fi

		# device entry
		elif [[ $line == $DEV ]]
		then
			device=`echo ${line:1:4} | tr "[:upper:]" "[:lower:]"`
			table_device=0x${line:1:4}
			dev_str=${line#${line:0:7}}
			ids_device=`echo ${ids_in:1:4} | tr "[:upper:]" "[:lower:]"`
			table_line="$vendor	$table_device	\"$driver\"	\"$ven_str|$dev_str\""
			# add entry to pci.ids
			exec 0<&3
			exec 1>&6
			while [[ $ids_in != $DEV ||
				 $ids_device < $device ]]
			do
				if [[ $ids_in == $VEN ]]
				then
					break
				fi
				if [[ $ids_device != ${ids_in:1:4} ]]
				then
					echo "${ids_in:0:1}$ids_device${ids_in#${ids_in:0:5}}"
				else
					echo "$ids_in"
				fi
				read -r ids_in
				ids_device=`echo ${ids_in:1:4} | tr "[:upper:]" "[:lower:]"`
			done
			if [[ $device != ${line:1:4} ]]
			then
				echo "${line:0:1}$device${line#${line:0:5}}"
			else
				echo "$line"
			fi
			if [[ $ids_device == $device ]]
			then
				read -r ids_in
			fi
			# add entry to pcitable
			if [ "$2" != "/dev/null" ];then
			exec 0<&4
			exec 1>&7
			while [[ $table_in != $TABLE_DEV ||
				 ${table_in:0:6} < $vendor ||
				 ( ${table_in:0:6} == $vendor &&
				   ${table_in:7:6} < $table_device ) ]]
			do
				echo "$table_in"
				read -r table_in
			done
			echo "$table_line"
			if [[ ${table_in:0:6} == $vendor &&
			      ${table_in:7:6} == $table_device ]]
			then
				read -r table_in
			fi
			fi
		# subsystem entry
		elif [[ $line == $SUB ]]
		then
			subven=`echo ${line:2:4} | tr "[:upper:]" "[:lower:]"`
			subdev=`echo ${line:7:4} | tr "[:upper:]" "[:lower:]"`
			table_subven=0x${line:2:4}
			table_subdev=0x${line:7:4}
			sub_str=${line#${line:0:13}}
			ids_subven=`echo ${ids_in:2:4} | tr "[:upper:]" "[:lower:]"`
			ids_subdev=`echo ${ids_in:7:4} | tr "[:upper:]" "[:lower:]"`
			table_line="$vendor	$table_device	$table_subven	$table_subdev	\"$driver\"	\"$ven_str|$sub_str\""
			# add entry to pci.ids
			exec 0<&3
			exec 1>&6
			while [[ $ids_in != $SUB ||
				 $ids_subven < $subven ||
				 ( $ids_subven == $subven && 
				   $ids_subdev < $subdev ) ]]
			do
				if [[ $ids_in == $VEN ||
				      $ids_in == $DEV ]]
				then
					break
				fi
				if [[ ! (${ids_in:2:4} == "1014" &&
					 ${ids_in:7:4} == "052C") ]]
				then
					if [[ $ids_subven != ${ids_in:2:4} || $ids_subdev != ${ids_in:7:4} ]]
					then
						echo "${ids_in:0:2}$ids_subven $ids_subdev${ids_in#${ids_in:0:11}}"
					else
						echo "$ids_in"
					fi
				fi
				read -r ids_in
				ids_subven=`echo ${ids_in:2:4} | tr "[:upper:]" "[:lower:]"`
				ids_subdev=`echo ${ids_in:7:4} | tr "[:upper:]" "[:lower:]"`
			done
			if [[ $subven != ${line:2:4} || $subdev != ${line:7:4} ]]
			then
				echo "${line:0:2}$subven $subdev${line#${line:0:11}}"
			else
				echo "$line"
			fi
			if [[ $ids_subven == $subven  &&
			      $ids_subdev == $subdev ]]
			then
				read -r ids_in
			fi
			# add entry to pcitable
			if [ "$2" != "/dev/null" ];then
			exec 0<&4
			exec 1>&7
			while [[ $table_in != $TABLE_SUB ||
				 ${table_in:14:6} < $table_subven ||
				 ( ${table_in:14:6} == $table_subven &&
				   ${table_in:21:6} < $table_subdev ) ]]
			do
				if [[ $table_in == $TABLE_DEV ]]
				then
					break
				fi
				if [[ ! (${table_in:14:6} == "0x1014" &&
					 ${table_in:21:6} == "0x052C") ]]
				then
					echo "$table_in"
				fi
				read -r table_in
			done
			echo "$table_line"
			if [[ ${table_in:14:6} == $table_subven &&
			      ${table_in:21:6} == $table_subdev ]]
			then
				read -r table_in
			fi
			fi
		fi

		exec 0<&5
	done

	# print the remainder of the original files
	exec 0<&3
	exec 1>&6
	echo "$ids_in"
	while read -r ids_in
	do
		echo "$ids_in"
	done

	if [ "$2" != "/dev/null" ];then
	exec 0>&4
	exec 1>&7
	echo "$table_in"
	while read -r table_in
	do
		echo "$table_in"
	done
	fi

	break
done <&5

exec 3<&-
exec 4<&-
exec 5<&-
exec 6>&-
exec 7>&-

END

mv -f $LD/pci.ids.new  %{pciids}
if [ "%{pcitable}" != "/dev/null" ]; then
mv -f $LD/pcitable.new %{pcitable}
fi

uname -r | grep BOOT || /sbin/depmod -a > /dev/null 2>&1 || true

#if [ -x "/usr/sbin/weak-modules" ]; then
#    modules=( $(cat %{_docdir}/%{pkg}/file.list | grep '\.ko$' | xargs realpath) )
#    printf '%s\n' "${modules[@]}" | /usr/sbin/weak-modules --no-initramfs --add-modules
#fi

echo "Updating initrd..."
# Decide which initrd update utility to use.
# Default is dracut but we'll try mkinitrd if that's not found.
which dracut >/dev/null 2>&1
if [ $? -eq 0 ]; then
	echo "Using dracut to update initrd..."
	initrd_cmd="dracut --force"
else
	which mkinitrd >/dev/null 2>&1
	if [ $? -eq 0 ]; then
		echo "Using mkinitrd to update initrd..."
		initrd_cmd="mkinitrd"
	else
		echo "Unable to find initrd update utility."
		echo "You must update your initrd for changes to take place."
		exit -1
	fi
fi

# Do the initrd update and report success or failure.
if [ "$initrd_cmd" != "" ]; then
	eval "$initrd_cmd"
	if [ $? -ne 0 ]; then
		echo "Failed to update initrd."
		echo "You must update your initrd for changes to take place."
		exit -1
	else
		echo "Successfully updated initrd."
	fi
fi

%preun
# save tmp list of installed kernel modules for weak-modules
cat %{_docdir}/kmod-%{pkg}/file.list | grep '\.ko.new$' | xargs realpath > /var/run/rpm-%{pkg}-modules.list

rm -rf /usr/local/share/%{pkg}

# If doing RPM un-install
if [ $1 -eq 0 ] ; then
	FL="%{_docdir}/%{pkg}-%{pkg_version}/file.list
    		%{_docdir}/%{pkg}/file.list"
	FL=$(for d in $FL ; do if [ -e $d ]; then echo $d; break; fi;  done)

	# Remove driver link
	for f in $(sed 's/\.new$//' $FL) ; do
		rm -f $f
	done

	# Restore old drivers
	if [ -d /usr/local/share/%{pkg} ]; then
		cd /usr/local/share/%{pkg}; find . -name '%{pkg}.*o*' -exec cp --parents {} /lib/modules/ \;
		cd /usr/local/share/%{pkg}; find . -name '%{pkg}_*.*o*' -exec cp --parents {} /lib/modules/ \;
		rm -rf /usr/local/share/%{pkg}
	fi
fi

%postun
depmod -a > /dev/null 2>&1

if [ -x "/usr/sbin/weak-modules" ]; then
    modules=( $(cat /var/run/rpm-%{pkg}-modules.list) )
    printf '%s\n' "${modules[@]}" | /usr/sbin/weak-modules --no-initramfs --remove-modules
fi
rm /var/run/rpm-%{pkg}-modules.list

# Update initramfs or initrd image
# Since this process varies across distributions,
# only support the simplest and most common methods. If
# we can't update the image automatically, we'll display a warning message
# indicating that the system administrator must perform some work manually.

if which dracut >/dev/null 2>&1; then
        echo "Updating initramfs with dracut..."
        if dracut --force ; then
                echo "Successfully updated initramfs."
        else
                echo "Failed to update initramfs."
                echo "You must update your initramfs image for changes to take place."
                exit -1
        fi
elif which mkinitrd >/dev/null 2>&1; then
        echo "Updating initrd with mkinitrd..."
        if mkinitrd; then
                echo "Successfully updated initrd."
        else
                echo "Failed to update initrd."
                echo "You must update your initrd image for changes to take place."
                exit -1
        fi
else
        echo "Unable to determine utility to update initrd image."
        echo "You must update your initrd manually for changes to take place."
        exit -1
fi

%changelog
* Fri Jul 19 2024 shengaoya <shengaoya@inspur.com> - 5.14.16
- Introduce 5.14.16 igb driver (shengaoya)
